/*
	Copyright 2009 Anatol Gregory Mayen
	
	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License. 
	You may obtain a copy of the License at 
	
	http://www.apache.org/licenses/LICENSE-2.0 
	
	Unless required by applicable law or agreed to in writing, software 
	distributed under the License is distributed on an "AS IS" BASIS, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
	See the License for the specific language governing permissions and 
	limitations under the License. 
*/
package eu.maydu.gwt.validation.client.validators.numeric;

import com.google.gwt.user.client.ui.HasValue;

import eu.maydu.gwt.validation.client.ValidationAction;
import eu.maydu.gwt.validation.client.ValidationResult;
import eu.maydu.gwt.validation.client.i18n.StandardValidationMessages;
import eu.maydu.gwt.validation.client.i18n.ValidationMessages;
import eu.maydu.gwt.validation.client.validators.StringValidator;

/**
 * 
 * @author Anatol Mayen
 *
 */
public class DoubleValidator<V extends HasValue<String>> extends StringValidator<DoubleValidator<V>, V> {

	private float min, max;
	private boolean noMinMax = false;
	
	public DoubleValidator(V text) {
		this(text, null);
	}
	
	public DoubleValidator(V text, String customMsgKey) {
		this(text, false);
		this.noMinMax = true;
		this.setCustomMsgKey(customMsgKey);
	}
	
	public DoubleValidator(V text, float min, float max) {
		this(text, min, max, false);
	}
	
	public DoubleValidator(V text, float min, float max, String customMsgKey) {
		this(text, min, max, false);
		this.setCustomMsgKey(customMsgKey);
	}
		
	public DoubleValidator(V text, boolean preventsPropagationOfValidationChain) {
		this(text, preventsPropagationOfValidationChain, null);
	}
	
	public DoubleValidator(V text, boolean preventsPropagationOfValidationChain, String customMsgKey) {
		super(text);
		this.setPreventsPropagationOfValidationChain(preventsPropagationOfValidationChain);
		this.noMinMax = true;
		this.setCustomMsgKey(customMsgKey);
	}
	
	public DoubleValidator(V text, float min, float max, boolean preventsPropagationOfValidationChain) {
		this(text, min, max, preventsPropagationOfValidationChain, null);
	}
	
	public DoubleValidator(V text, float min, float max, boolean preventsPropagationOfValidationChain, String customMsgKey) {
		super(text);
		this.setPreventsPropagationOfValidationChain(preventsPropagationOfValidationChain);
		this.min = min;
		this.max = max;
		this.noMinMax = false;
		this.setCustomMsgKey(customMsgKey);
	}
	
	@Override
	public ValidationResult validate(ValidationMessages allMessages) {
		StandardValidationMessages messages = allMessages.getStandardMessages();
		String str = valueHolder.getValue();
		if(str == null)
			str = "";
		str = str.trim();
		if(str.equals("") && isRequired())
			return new ValidationResult(getErrorMessage(allMessages, messages.notAnInteger()));
		else if(str.equals(""))
			return null;
		
		try {
			double value = Double.parseDouble(str);
			if(!this.noMinMax) {
				if(value < this.min || value > this.max) {
					return new ValidationResult(getErrorMessage(allMessages, messages.notInRange(this.min, this.max, value),this.min, this.max, value));
				}
			}
		}catch(NumberFormatException ex) {
			return new ValidationResult(getErrorMessage(allMessages, messages.notADouble()));
		}
		
		return null;
	}
	
	public void invokeActions(ValidationResult result) {
		for(ValidationAction<V> action : getFailureActions()) {
			action.invoke(result, valueHolder);
		}
	}

}
