/*
	Copyright 2009 Anatol Gregory Mayen
	
	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License. 
	You may obtain a copy of the License at 
	
	http://www.apache.org/licenses/LICENSE-2.0 
	
	Unless required by applicable law or agreed to in writing, software 
	distributed under the License is distributed on an "AS IS" BASIS, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
	See the License for the specific language governing permissions and 
	limitations under the License. 
*/
package eu.maydu.gwt.validation.client.validators.numeric;

import com.google.gwt.user.client.ui.HasValue;

import eu.maydu.gwt.validation.client.ValidationAction;
import eu.maydu.gwt.validation.client.ValidationResult;
import eu.maydu.gwt.validation.client.i18n.StandardValidationMessages;
import eu.maydu.gwt.validation.client.i18n.ValidationMessages;
import eu.maydu.gwt.validation.client.validators.StringValidator;

/**
 * 
 * @author Anatol Mayen
 *
 */
public class IntegerValidator<V extends HasValue<String>> extends StringValidator<IntegerValidator<V>, V> {

	private int min, max;
	private boolean noMinMax = false;
	
	public IntegerValidator(V text) {
		this(text, null);
	}
	
	public IntegerValidator(V text, String customMsgKey) {
		this(text, false);
		this.noMinMax = true;
		
	}
	
	public IntegerValidator(V text, int min, int max) {
		this(text, min, max, false);
	}
	
	public IntegerValidator(V text, int min, int max, String customMsgKey) {
		this(text, min, max, false);
		this.setCustomMsgKey(customMsgKey);
	}
	
	public IntegerValidator(V text, boolean preventsPropagationOfValidationChain) {
		this(text, preventsPropagationOfValidationChain, null);
	}
	
	public IntegerValidator(V text, boolean preventsPropagationOfValidationChain, String customMsgKey) {
		super(text);
		this.setPreventsPropagationOfValidationChain(preventsPropagationOfValidationChain);
		if(text == null)
			throw new RuntimeException("text must not be null");
		this.noMinMax = true;
		this.setCustomMsgKey(customMsgKey);
	}
	
	public IntegerValidator(V text, int min, int max, boolean preventsPropagationOfValidationChain) {
		this(text, min, max, preventsPropagationOfValidationChain, null);
		
	}
	
	public IntegerValidator(V text, int min, int max, boolean preventsPropagationOfValidationChain, String customMsgKey) {
		super(text);
		this.setPreventsPropagationOfValidationChain(preventsPropagationOfValidationChain);
		if(text == null)
			throw new RuntimeException("text must not be null");
		this.min = min;
		this.max = max;
		this.noMinMax = false;
		this.setCustomMsgKey(customMsgKey);
	}
	
	@Override
	public ValidationResult validate(ValidationMessages allMessages) {
		StandardValidationMessages messages = allMessages.getStandardMessages();
		String str = valueHolder.getValue();
		if(!isRequired() && str.equals(""))
			return null;
		str = str.trim();
		if(str.equals(""))
			return new ValidationResult(getErrorMessage(allMessages, messages.notAnInteger()));
		if(!this.noMinMax) {
			//Integer in range
			try {
				long value = Long.parseLong(str);
				if(value < this.min || value > this.max) {
					return new ValidationResult(getErrorMessage(allMessages, messages.notInRange(this.min, this.max, value), this.min, this.max, value));
				}
			}catch(NumberFormatException ex) {
				return new ValidationResult(getErrorMessage(allMessages, messages.notAnInteger()));
			}
		}else {
			//Any integer
			try {
				long value = Long.parseLong(str);
				if(value < Integer.MIN_VALUE || value > Integer.MAX_VALUE)
					return new ValidationResult(getErrorMessage(allMessages, messages.notAnInteger()));
			}catch(NumberFormatException ex) {
				return new ValidationResult(getErrorMessage(allMessages, messages.notAnInteger()));
			}
		}
		
		return null;
	}
	
	public void invokeActions(ValidationResult result) {
		for(ValidationAction<V> action : getFailureActions()) {
			action.invoke(result, valueHolder);
		}
	}

}
