/*
	Copyright 2009 Anatol Gregory Mayen
	
	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License. 
	You may obtain a copy of the License at 
	
	http://www.apache.org/licenses/LICENSE-2.0 
	
	Unless required by applicable law or agreed to in writing, software 
	distributed under the License is distributed on an "AS IS" BASIS, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
	See the License for the specific language governing permissions and 
	limitations under the License. 
*/
package eu.maydu.gwt.validation.client.validators.strings;

import com.google.gwt.user.client.ui.HasValue;

import eu.maydu.gwt.validation.client.ValidationAction;
import eu.maydu.gwt.validation.client.ValidationResult;
import eu.maydu.gwt.validation.client.i18n.StandardValidationMessages;
import eu.maydu.gwt.validation.client.i18n.ValidationMessages;
import eu.maydu.gwt.validation.client.validators.StringValidator;


/**
 * Validates that a given input is in the given length range.
 *  
 * @author Anatol Mayen
 *
 */
public class StringLengthValidator<V extends HasValue<String>> extends StringValidator<StringLengthValidator<V>, V> {

	protected int min, max;
	
	public StringLengthValidator(V text) {
		this(text, false);
	}
	
	public StringLengthValidator(V text, int min, int max) {
		this(text, min, max, false);
	}
	
	
	public StringLengthValidator(V text, boolean preventsPropagationOfValidationChain) {
		this(text, preventsPropagationOfValidationChain, null);
	}
	
	public StringLengthValidator(V text, boolean preventsPropagationOfValidationChain, String customMsgKey) {
		super(text);
		this.setPreventsPropagationOfValidationChain(preventsPropagationOfValidationChain);
		if(text == null)
			throw new IllegalArgumentException("text must not be null");
		this.setCustomMsgKey(customMsgKey);
	}
	
	public StringLengthValidator(V text, int min, int max, boolean preventsPropagationOfValidationChain) {
		this(text, min, max, preventsPropagationOfValidationChain, null);
	}
	
	public StringLengthValidator(V text, int min, int max, boolean preventsPropagationOfValidationChain, String customMsgKey) {
		super(text);
		this.setPreventsPropagationOfValidationChain(preventsPropagationOfValidationChain);
		if(text == null)
			throw new IllegalArgumentException("text must not be null");
		setMin(min).setMax(max);
		this.setCustomMsgKey(customMsgKey);
	}
	
	@Override
	public ValidationResult validate(ValidationMessages allMessages) {
		StandardValidationMessages messages = allMessages.getStandardMessages();
		String str = valueHolder.getValue();
		if(str == null)
			str = "";
		if(str.equals("") && !isRequired())
			return null;
		if(str.length() < this.min || str.length() > this.max)
			return new ValidationResult(getErrorMessage(allMessages, messages.length(this.min, this.max, str.length()), this.min, this.max, str.length()));

		return null;
	}
	
	public void invokeActions(ValidationResult result) {
		for(ValidationAction<V> action : getFailureActions()) {
			action.invoke(result, valueHolder);
		}
	}
	
	public StringLengthValidator<V> setMin(int min) {
		this.min = min;
		return this;
	}
	
	public StringLengthValidator<V> setMax(int max) {
		this.max = max;
		return this;
	}

}
